# flcheck --
#
# This file implements package flcheck, a fusion logic modelchecker 
# using the CUDD BDD package
#
# Copyright (C) 2009, 2010  Antonio Cau

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# FL derived
#####################################

# for historical reason and is defined
proc and { a b } {
    return "{ {$a} and {$b} }"
}

# for historical reason false is defined
proc false {} {
   return "false"
}

proc implies {a b} {
    return "not{$a} or {$b}"
}

proc equiv {a b} {
    set x "not{$a}"
    set y "not{$b}"
    return "{{$a and $b} or {$x and $y}}"
}

proc l_and {alist} {
    set x "true"
    foreach e $alist {
        set x "{ {$x} and {$e} }"
    }
    return $x
}


proc tl_and {alist} {
    set x "true"
    foreach e $alist {
        set x "{and $x $e}"
    }
    return $x
}

#############
# derived fusion expressions

proc len_e { n } {
    if {$n < 0 } { 
        return "test{false}"
    } else {
      if {$n == 0 } {
          return "test{true}"
      } else {
          return "step{true};[len_e [incr n -1]]"
      }
    }
}

proc true_e {} {
    return "chopstar{step{true}}"
}

proc sometimes_e { a } {
    return "[true_e];{$a}"
}

proc skip_e {} {
    return "step{true}"
}

proc empty_e {} {
    return "test{true}"
}

########################
# derived right fusion logic

proc len_r { n } {
    return "<[len_e $n]>[empty_r]"
}

proc next_r {a} {
    return "<[skip_e]>{$a}"
}

proc sometimes_r { a } { 
    return "<[true_e]>{$a}"
}

proc always_r { a } {
    return "not<[true_e]>{not{$a}}"
}

proc di_r { a } {
    return "<[true_e];test{$a}>true"
} 

proc bi_r { a } {
    return "not [di_r {not{$a}}]"
}

proc more_r {} {    
    return "<[skip_e]>{true}"
}

proc empty_r {} {    
    return "not{[more_r]}"
}

proc skip_r {} {    
    return "<[skip_e]>{[empty_r]}"
}

#proc finite_r {} {
#    return "<[finite_e]>{[empty_r]}"
#}

proc fin_r { a } {
    return "[always_r [implies [empty_r] $a]]"
}

########################
# derived left fusion logic


proc len_l { n } {
    return "[empty_l] <[len_e $n]>"
}

proc more_l {} {
    return "true <[skip_e]>"
}

proc empty_l {} {    
    return "{not{[more_l]}}"
}

proc skip_l {} {    
    return "{[empty_l]}<[skip_e]>"
}

proc sometimes_l { a } { 
    return "true <test{$a};[true_e]>"
}

proc always_l { a } {
    return "not [sometimes_l not{$a} ]"
}

proc di_l { a } {
    return "{$a}<[true_e]>"
} 

proc bi_r { a } {
    return "not [di_l {not{$a}}]"
}

proc fin_l { a } {
    return "true < [true_e];test{$a}>"
}